# -*- coding: utf-8 -*-
'''
    The venvm is small webtool for manager virtualenv.
    Copyright (C) 2013  Rodrigo Pinheiro Matias <rodrigopmatias@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see {http://www.gnu.org/licenses/}.
'''
from flask import Flask, render_template, session, request
from datetime import timedelta

import sys
import os
import re
import subprocess as sp
import venvm
import json
import codecs

router_dir = os.path.dirname(os.path.abspath(__file__))
i18n_dir = os.path.join(router_dir, 'i18n')

app = Flask(__name__)

app.permanent_session_lifetime = timedelta(minutes=10)
app.secret_key = '123'

DEFINED_MENU = {
    'actions': [
        {
            'href': 'manager',
            'title': u'Gerenciar Pacotes'
        },
        {
            'href': 'information',
            'title': u'Informações'
        }
    ],
    'sites': [
        {
            'href': 'python',
            'title': u'Python'
        },
        {
            'href': 'pypy',
            'title': u'Pypy'
        },
        {
            'href': 'pypi',
            'title': u'Python Packages'
        }
    ]
}


def get_language():
    lang = session.get('lang', None)
    session.permanent = True

    if lang is None:
        for item in request.accept_languages.values():
            if os.path.exists(get_i18n_filepath(item)):
                lang = item
                break
        lang = 'pt-BR' if lang is None else lang
        session['lang'] = lang

    return lang


def get_i18n_filepath(lang):
    return os.path.join(i18n_dir, '%s.json' % lang)


def load_i18n(lang):
    db = {}
    try:
        with codecs.open(get_i18n_filepath(lang)) as fd:
            db = json.load(fd)
    except:
        db = {}
    finally:
        return db


def _(text):
    lang = get_language()
    db = load_i18n(lang)

    translate = db.get(text, None)

    if translate is None:
        print('I18N(%s) -> "%s": "%s",' % (lang, text, text))

    return translate if translate is not None else text

app.jinja_env.filters['_'] = _


def get_languages_options():
    regex = re.compile('^(?P<lang>.*)\.json$')
    obj = []

    for filename in os.listdir(i18n_dir):
        rst = regex.match(filename)
        if rst is not None:
            lang = rst.groupdict().get('lang')
            db = load_i18n(lang)
            obj.append({
                'href': lang,
                'title': db.get('language', 'undefined')
            })

    return obj


@app.route('/')
def index():
    params = {}

    params.update(
        menu=DEFINED_MENU,
        langs=get_languages_options()
    )
    return render_template('index.html', **params)


@app.route('/language/<lang>')
def language(lang):
    rst = {'success': False}

    if os.path.exists(get_i18n_filepath(lang)) is True:
        rst.update(success=True)
        session.update(lang=lang)

    return json.dumps(rst)


@app.route('/manager')
def manager():
    return render_template('manager.html')


def get_directory_size(directory):
    pid = sp.Popen(['/usr/bin/du', '-sh', directory], stdout=sp.PIPE)
    regex = re.compile('^(?P<size>\d+(M|G|T|H)).*$')

    rst = regex.match(pid.stdout.read())
    if rst is not None:
        return rst.groupdict().get('size')
    else:
        return 'undefined'


def get_virtualenv_version():
    pid = sp.Popen('type -p virtualenv', shell=True, stdout=sp.PIPE)
    path = pid.stdout.read().replace('\n', '')

    if os.path.isfile(path) and os.access(path, os.X_OK):
        pid = sp.Popen([path, '--version'], stdout=sp.PIPE)
        return pid.stdout.read().replace('\n', '')
    else:
        return 'undefined'


@app.route('/information')
def information():
    params = {
        'py_version': '%s.%s.%s' % (
            sys.version_info.major,
            sys.version_info.minor,
            sys.version_info.micro
        ),
        'py_platform': sys.platform,
        'py_api_version': sys.api_version,
        'py_copyright': sys.copyright,
        'venv_version': get_virtualenv_version(),
        'venvm_version': venvm.__version__,
        'venvm_codename': venvm.__codename__,
        'size': get_directory_size(os.environ.get('VIRTUAL_ENV')),
        'path': os.environ.get('VIRTUAL_ENV')
    }

    try:
        import pip
    except Exception:
        pass
    else:
        params.update(
            pip={
                'version': pip.__version__
            }
        )

    try:
        import setuptools
    except Exception:
        pass
    else:
        params.update(
            setuptools={
                'version': setuptools.__version__
            }
        )

    try:
        import distutils
    except Exception:
        pass
    else:
        params.update(
            distutils={
                'version': distutils.__version__
            }
        )

    return render_template('information.html', **params)
